module smoothing
   implicit none

contains

   subroutine smooth(v, k)
      real,intent(inout) :: v(:)
      integer,intent(in) :: k
      integer :: n, l, i, j
      real :: work(size(v))
      n=size(v)
      l = 2*k +1
      work = 0.0
      do i=1,n
         do j=i-k,i+k
            work(i) = work(i) + v(1+mod(n-1+j, n))
         enddo
      enddo

      v = work/l

   end subroutine smooth
end module smoothing

program test_smooth
   use smoothing
   implicit none
   integer, parameter :: n=10000000
   integer :: i, k
   real :: x(n)
   real :: t1,t2

   k = 10
   if ( k > n) stop 'More smoothing points than array elements'

   do i = 1, n
      x(i) = real(mod(i,n/2))
   enddo

   call cpu_time(t1)
   call smooth(x,k)
   call cpu_time(t2)

   write(*,*) 'CPU time:  ',t2-t1, 'seconds'
end program test_smooth
